import sys
sys.path.append('.')

import requests

import sigan.utils.display as display
import sigan.utils.general_utils as general_utils
from sigan.config import get_config

cfg = get_config()


def create_alarm(data: dict):
    
    resp = requests.post(
        url=cfg.base_url + "/slack",
        json=data,
    )
    if resp.status_code == 201:
        display.center_print("Alarm added successfully!", type="success")
        alarm = general_utils.bytes2dict(resp.content)
        return alarm
    elif resp.status_code == 400:
        detail = general_utils.bytes2dict(resp.content)['detail']
        display.center_print(detail, type='detail')
    else:
        display.center_print("Error occurred", type='detail')
    exit(0)


def delete_alarm(data: dict):
    
    resp = requests.post(
        cfg.base_url + "/slack/delete",
        json=data,
    )
    if resp.status_code == 200:
        alarm = general_utils.bytes2dict(resp.content)
        display.center_print("Alarm added successfully!", type="success")
        return alarm
    elif resp.status_code == 400:
        detail = general_utils.bytes2dict(resp.content)['detail']
        display.center_print(detail, type='detail')
    else:
        display.center_print("Error occurred", type='detail')
    exit(0)


def get_all_alarm(data: dict):
    
    resp = requests.get(
        cfg.base_url + "/slack",
        json=data,
    )
    
    if resp.status_code == 200:
        return general_utils.bytes2dict(resp.content)['alarm']
    elif resp.status_code == 400:
        detail = general_utils.bytes2dict(resp.content)['detail']
        display.center_print(detail, type='detail')
    else:
        display.center_print("Error occurred", type='detail')
    exit(0)


def change_content(data: dict):
    resp = requests.patch(
        cfg.base_url + "/slack/change_content",
        json=data
    )
    if resp.status_code == 200:
        display.center_print("Alarm content changed successfully!", type="success")
        return general_utils.bytes2dict(resp.content)
    elif resp.status_code == 400:
        detail = general_utils.bytes2dict(resp.content)['detail']
        display.center_print(detail, type='detail')
    else:
        display.center_print("Error occurred", type='detail')
    exit(0)
        
        
def change_deadline(data: dict):
    resp = requests.patch(
        cfg.base_url + "/slack/change_deadline",
        json=data
    )
    if resp.status_code == 200:
        display.center_print("Alarm deadline changed successfully!", type="success")
        return general_utils.bytes2dict(resp.content)
    elif resp.status_code == 400:
        detail = general_utils.bytes2dict(resp.content)['detail']
        display.center_print(detail, type='detail')
    else:
        display.center_print("Error occurred", type='detail')
    exit(0)


def change_date(data: dict):
    resp = requests.patch(
        cfg.base_url + "/slack/change_date",
        json=data
    )
    if resp.status_code == 200:
        display.center_print("Alarm date changed successfully!", type="success")
        return general_utils.bytes2dict(resp.content)
    elif resp.status_code == 400:
        detail = general_utils.bytes2dict(resp.content)['detail']
        display.center_print(detail, type='detail')
    else:
        display.center_print("Error occurred", type='detail')
    exit(0)

        
def change_interval(data: dict):
    resp = requests.patch(
        cfg.base_url + "/slack/change_interval",
        json=data
    )
    if resp.status_code == 200:
        display.center_print("Alarm interval changed successfully!", type="success")
        return general_utils.bytes2dict(resp.content)
    elif resp.status_code == 400:
        detail = general_utils.bytes2dict(resp.content)['detail']
        display.center_print(detail, type='detail')
    else:
        display.center_print("Error occurred", type='detail')
    exit(0)
    
    
def register(data: dict):
    resp = requests.post(
        cfg.base_url + "/slack/register",
        json=data
    )
    if resp.status_code == 200:
        display.center_print("Register successfully!", type="success")
        return general_utils.bytes2dict(resp.content)
    elif resp.status_code == 400:
        detail = general_utils.bytes2dict(resp.content)['detail']
        display.center_print(detail, type='detail')
    else:
        display.center_print("Error occurred", type='detail')
    exit(0)


import typer

import sys
sys.path.append('.')

import sigan.utils.general_utils as general_utils
import sigan.commands.command as command

from sigan.config import get_config


cfg = get_config()

app = typer.Typer(rich_markup_mode='rich', help="Enter [red]girok <command name> --help[/red] to see more detailed documentations of commands!")
app.registered_commands.extend(command.app.registered_commands)

def version_callback(value: bool):
    if value:
        print(f"{cfg.version}")
        raise typer.Exit()


@app.callback()
def pre_command_callback(ctx: typer.Context, version: bool = typer.Option(None, "--version", callback=version_callback, is_eager=True)):
    cmd = ctx.invoked_subcommand
    
    general_utils.config_setup(cfg.app_dir, cfg.config_path)

    if cmd == "add" or cmd == "show" or cmd == "delete" or cmd == "chcnt" or cmd == "chdate" or cmd == "chdl" or cmd == "chinv":
        try:
            user_info = general_utils.read_team_id(cfg.app_dir)
        except FileNotFoundError:
            raise typer.BadParameter("Registration is not complete.")


from datetime import datetime, timedelta


def is_valid_hour(hour: int):
    if hour < 0 or hour > 23:
        return False
    return True


def is_valid_minute(minute: int):
    if minute < 0 or minute > 59:
        return False
    return True


def change_deadline_to_date(deadline: str):
    year, month, day = datetime.now().year, None, None
    date_list = list(map(int, deadline.split('/')))
    year, month, day = date_list

    return year, month, day


def change_datetime_to_date(alarm_date: str):
    year, month, day = datetime.now().year, None, None
    date_list = list(map(int, alarm_date.split('-')))
    year, month, day = date_list
    
    return year, month, day


def get_date_from_shortcut(interval_day: list, time: str):
    alarm_date = f"{datetime.now().year}-{datetime.now().month}-{datetime.now().day} {time}:00"
    alarm_date = datetime.strptime(alarm_date, '%Y-%m-%d %H:%M:%S')

    if "everyday" in interval_day:
        if alarm_date < datetime.now():
            alarm_date += timedelta(days=1)
        return alarm_date.year, alarm_date.month, alarm_date.day
    
    weekday_mapping = {'mon': 0, 'tue': 1, 'wed': 2, 'thu': 3, 'fri': 4, 'sat': 5, 'sun': 6}  
    current_weekday_list = []
    
    current_weekday_num = datetime.now().weekday()
    
    for interval in interval_day:
        day_offset = weekday_mapping[interval] - current_weekday_num
        current_weekday_list.append(day_offset)
    
    # 0에서 가장 가까운 양수부터 음수 순으로 정렬
    current_weekday_list = sorted(current_weekday_list, key=lambda x: (abs(x), -x))

    if current_weekday_list[0] < 0:
        alarm_date += timedelta(days=current_weekday_list[-1]+7)
        return alarm_date.year, alarm_date.month, alarm_date.day
    
    for day_offset in current_weekday_list:        
        if alarm_date + timedelta(days=day_offset) > datetime.now():
            alarm_date += timedelta(days=day_offset)
            return alarm_date.year, alarm_date.month, alarm_date.day
        else:
            date_offset = alarm_date + timedelta(days=7+day_offset)
    
    return date_offset.year, date_offset.month, date_offset.day



import os
import typer
from pathlib import Path
from functools import lru_cache

import sys
sys.path.append('.')


import sigan.utils.general_utils as general_utils


cur_file_dir = os.path.dirname(os.path.realpath(__file__))

class Config:
    def __init__(self):
        self.config = general_utils.read_json(os.path.join(cur_file_dir, "config.json"))
        self.base_url = self.config['base_url']
        self.app_name = self.config['app_name']
        self.app_dir = typer.get_app_dir(self.app_name)
        self.app_dir_path: Path = Path(self.app_dir)
        self.team_id_path: Path = Path(self.app_dir) / "team_id_cache.json"
        self.config_path: Path = Path(self.app_dir) / "config.json"
        self.version = self.config['version']
        
        
@lru_cache()
def get_config():
    return Config()


import json, os
from pathlib import Path

import sys
sys.path.append('.')


def sql_obj_to_dict(sql_obj):
    d = dict()
    for col in sql_obj.__table__.columns:
        d[col.name] = getattr(sql_obj, col.name)
    return d


def sql_obj_list_to_dict_list(sql_obj_list):
    return [sql_obj_to_dict(sql_obj) for sql_obj in sql_obj_list]


def bytes2dict(b):
    return json.loads(b.decode('utf-8'))


def config_setup(APP_DIR, APP_CACHE_PATH):
    if not APP_CACHE_PATH.is_file():
        if not os.path.isdir(APP_DIR):
            os.makedirs(APP_DIR)
        # APP_DIR_PATH.mkdir(parents=True, exist_ok=True)
            

def read_json(fpath):
    with open(fpath, 'r') as f:
        data = json.load(f)
        return data
    
def write_json(fpath, data):
    with open(fpath, 'w') as f:
        json.dump(data, f)
        

def cache_team_id(app_dir, cache: dict):
    target_path = Path(app_dir) / 'team_id_cache.json'
    write_json(target_path, cache)
    

def read_team_id(app_dir):
    target_path = Path(app_dir) / 'team_id_cache.json'    
    return read_json(target_path)   


def cache_alarm_id(app_dir, cache: dict):
    target_path = Path(app_dir) / 'alarm_id_cache.json'
    write_json(target_path, cache)
    

def read_alarm_id(app_dir, alarm_id: int):
    target_path = Path(app_dir) / 'alarm_id_cache.json'
    data = read_json(target_path)
    for key in data.keys():
        if key == str(alarm_id):
            return data[str(alarm_id)]

    return None
